//
//  MegaTableView.m
//  MegaTableView
//
//  Created by CHENG POWEN on 13/3/11.
//  Copyright (c) 2013年 CHENG POWEN. All rights reserved.
//

#import "MegaTableView.h"

@implementation MegaTableView

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        [[self layer]setBorderColor:[[UIColor blackColor] CGColor]];
        [[self layer]setBorderWidth:2.0f];
        //[[self layer] setCornerRadius:5.0f];
        _defaultBGColor = [UIColor clearColor];
        _titleBGColor = _defaultBGColor;
        _sectionBGColor = _defaultBGColor;
        _cellBGColor = _defaultBGColor;
        _headerBGColor = _defaultBGColor;
        _defaultTextColor = [UIColor blackColor];
        _cellTextColor = _defaultTextColor;
        _titleTextColor = _defaultTextColor;
        _sectionTextColor = _defaultTextColor;
        _headerTextColor = _defaultTextColor;
        _defaultFont = [UIFont fontWithName:@"Helvetica" size:16.0f];
        _cellFont = _defaultFont;
        _titleFont = _defaultFont;
        _sectionFont = _defaultFont;
        _headerFont = _defaultFont;
        _defaultLineColor = [UIColor blackColor];
        _lineColr = _defaultLineColor;
        _defaultLineWidth = 1.0f;
        _lineWidth = _defaultLineWidth;
        _defaultselectionColor = [UIColor blueColor];
        _selectionColor = _defaultselectionColor;
        _sectionStartRow = 0;
        _sectionStep = 5;
        _defaultHeight = 44.0f;
        // Initialization code
    }
    return self;
}

#pragma mark - public
- (void)reloadMegaTable{
    [self reloadAllData];
    [self setBGColor];
    [self setTextColor];
    [self setFont];
    [self setLine];
    [self setSection];
    [self setSelectionColor];
    [self reloadColWidth];
    [self drawTable];
    [_headerTableView reloadData];
    [_contentTableView reloadData];
}

#pragma mark - dataSource
- (void)setDataSource:(id<MegaTableViewDataSource>)dataSource{
    if (_dataSource!=dataSource) {
        _dataSource = dataSource;
        respondsToWidthOfHeader = [dataSource respondsToSelector:@selector(widthOfHeaderInMegaTableView:)];
        respondsToTitleBGColorInMegaTableView = [_dataSource respondsToSelector:@selector(titleBGColorInMegaTableView:)];
        respondsToSectionBGColorInMegaTableView = [_dataSource respondsToSelector:@selector(sectionBGColorInMegaTableView:)];
        respondsToCellBGColorInMegaTableView = [_dataSource respondsToSelector:@selector(cellBGColorInMegaTableView:)];
        respondsToHeaderBGColorInMegaTableView = [_dataSource respondsToSelector:@selector(headerBGColorInMegatableView:)];
        respondsToTitleTextColorInMegaTableView = [_dataSource respondsToSelector:@selector(titleTextColorInMegaTableView:)];
        respondsToSectionTextColorInMegaTableView = [_dataSource respondsToSelector:@selector(sectionTextColorInMegaTableView:)];
        respondsToCellTextColorInMegaTableView = [_dataSource respondsToSelector:@selector(cellTextColorInMegaTableView:)];
        respondsToHeaderTextColorInMegaTableView = [_dataSource respondsToSelector:@selector(headerTextColorInMegatableView:)];
        respondsToTitleFontInMegaTableView = [_dataSource respondsToSelector:@selector(titleFontInMegaTableView:)];
        respondsToSectionFontInMegaTableView = [_dataSource respondsToSelector:@selector(sectionFontInMegaTableView:)];
        respondsToCellFontInMegaTableView = [_dataSource respondsToSelector:@selector(cellFontInMegaTableView:)];
        respondsToHeaderFontInMegaTableView = [_dataSource respondsToSelector:@selector(headerFontInMegaTableView:)];
        respondsToTableLineColorInMegaTableView = [_dataSource respondsToSelector:@selector(tableLineColorInMegatableView:)];
        respondsToTableLineWidthInMegaTableView = [_dataSource respondsToSelector:@selector(tableLineWidthInMegatableView:)];
        respondsToSectionStructInMegatableView = [_dataSource respondsToSelector:@selector(sectionStructInMegatableView:)];
        respondsToSelectionColorInMegatableView = [_dataSource respondsToSelector:@selector(selectionColorInMegatableView:)];
    }
}

#pragma mark - tableView dataSource
- (int)numberOfSectionsInTableView:(UITableView *)tableView{
    return 1;
}

- (int)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section{
    return [_dataSource numberOfRowsInMegaTableView:self];
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
    return _defaultHeight;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath{
    return _defaultHeight;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath{
    if (tableView==_contentTableView) {
        static NSString *cellID = @"contentCell";
        UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellID];
        if (cell==nil) {
            cell = [[UITableViewCell alloc]initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellID];
            [cell setSelectionStyle:UITableViewCellSelectionStyleNone];
        }
        NSArray *columnsInRow = [_columnsInRowsInContent objectAtIndex:indexPath.row];
        CGPoint point = CGPointMake(0, 0);
        UIColor *bgColor;
        UIFont *textFont;
        if ((indexPath.row-_sectionStartRow)%_sectionStep==0) {
            bgColor = _sectionBGColor;
            textFont = _sectionFont;
        }else {
            bgColor = _cellBGColor;
            textFont = _cellFont;
        }
        for (int i=0; i<[columnsInRow count]; i++) {
            if ([cell.contentView viewWithTag:10+i]) {
                [[cell.contentView viewWithTag:10+i] removeFromSuperview];
            }
            NSString *str = [columnsInRow objectAtIndex:i];
            float width = [[_colWidthForContent objectAtIndex:i] floatValue];
            CGRect frame = CGRectMake(point.x, point.y, width, _defaultHeight);
            point.x += width;
            UILabel *label = [self cellWithStr:str font:textFont textColor:_cellTextColor bgColor:bgColor frame:frame isGesture:YES];
            [label setTag:10+i];
            [cell.contentView addSubview:label];
            [cell addBottomLineWithWidth:_lineWidth color:_lineColr];
        }
        return cell;
    }else {
        static NSString *cellID = @"headerCell";
        UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellID];
        if (cell==nil) {
            cell = [[UITableViewCell alloc]initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellID];
            [cell setSelectionStyle:UITableViewCellSelectionStyleNone];
        }
        NSArray *columnsInRow = [_columnsInRowsInHeader objectAtIndex:indexPath.row];
        CGPoint point = CGPointMake(0, 0);
        UIColor *bgColor;
        UIFont *textFont;
        if ((indexPath.row-_sectionStartRow)%_sectionStep==0) {
            bgColor = _sectionBGColor;
            textFont = _sectionFont;
        }else {
            bgColor = _cellBGColor;
            textFont = _cellFont;
        }
        for (int i=0; i<[columnsInRow count]; i++) {
            if ([cell.contentView viewWithTag:10+i]) {
                [[cell.contentView viewWithTag:10+i] removeFromSuperview];
            }
            NSString *str = [columnsInRow objectAtIndex:i];
            float width = [[_colWidthForHeader objectAtIndex:i] floatValue];
            CGRect frame = CGRectMake(point.x, point.y, width, _defaultHeight);
            point.x += width;
            UILabel *label = [self cellWithStr:str font:textFont textColor:_cellTextColor bgColor:bgColor frame:frame isGesture:YES];
            [label setTag:10+i];
            [cell.contentView addSubview:label];
            [cell addBottomLineWithWidth:_lineWidth color:_lineColr];
        }
        return cell;
    }
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section{
    UIView *titleView = [[UIView alloc]init];
    if (tableView==_contentTableView) {
        CGPoint point = CGPointMake(0, 0);
        for (int i=0; i<[_titleInContent count]; i++) {
            NSString *titleStr = [_titleInContent objectAtIndex:i];
            float width = [[_colWidthForContent objectAtIndex:i] floatValue];
            CGRect frame = CGRectMake(point.x, point.y, width, _defaultHeight);
            point.x += width;
            UILabel *label = [self cellWithStr:titleStr font:_titleFont textColor:_titleTextColor bgColor:_titleBGColor frame:frame isGesture:YES];
            [titleView addSubview:label];
            [titleView addVerticalLineWithWidth:_lineWidth color:_lineColr atX:point.x];
            [titleView addBottomLineWithWidth:_lineWidth color:_lineColr];
        }
    }else {
        CGPoint point = CGPointMake(0, 0);
        for (int i=0; i<[_titleInHeader count]; i++) {
            NSString *titleStr = [_titleInHeader objectAtIndex:i];
            float width = [[_colWidthForHeader objectAtIndex:i]floatValue];
            CGRect frame = CGRectMake(point.x, point.y, width, _defaultHeight);
            point.x += width;
            UILabel *label = [self cellWithStr:titleStr font:_headerFont textColor:_headerTextColor bgColor:_headerBGColor frame:frame isGesture:YES];
            [titleView addSubview:label];
            [titleView addVerticalLineWithWidth:_lineWidth color:_lineColr atX:point.x];
            [titleView addBottomLineWithWidth:_lineWidth color:_lineColr];
        }
    }
    return titleView;
}

#pragma mark - tableview(scrollView) delegate
- (void)scrollViewDidScroll:(UIScrollView *)scrollView{
    UIScrollView *target;
    if (scrollView == _contentTableView) {
        target = _headerTableView;
    } else {
        target = _contentTableView;
    }
    target.contentOffset = scrollView.contentOffset;
}

#pragma mark - private

- (void)initAllTableView{
    CGFloat width = self.bounds.size.width;
    CGFloat height = self.bounds.size.height;
    CGFloat contentWidth = 0;
    CGFloat headerWidth = 0;
    for (NSNumber *num in _colWidthForContent) {
        contentWidth += [num floatValue];
    }
    for (NSNumber *num in _colWidthForHeader) {
        headerWidth += [num floatValue];
    }
    if (respondsToWidthOfHeader) {
        headerWidth= [ _dataSource widthOfHeaderInMegaTableView:self];
        _headerTableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, headerWidth, self.bounds.size.height) style:UITableViewStylePlain];
        
        CGPoint originOfSclView = CGPointMake(headerWidth, 0);
        CGSize sizeOfSclView = CGSizeMake(width-headerWidth, height);
        _sclView = [[UIScrollView alloc] initWithFrame:CGRectMake(originOfSclView.x, originOfSclView.y, sizeOfSclView.width, sizeOfSclView.height)];
        [_sclView setContentSize:CGSizeMake(contentWidth, sizeOfSclView.height)];
        [_sclView setBounces:NO];
        [_sclView setDirectionalLockEnabled:YES];
        
        _contentTableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, _sclView.contentSize.width, _sclView.contentSize.height) style:UITableViewStylePlain];
    }else {
        _headerTableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, headerWidth, self.bounds.size.height) style:UITableViewStylePlain];
        
        CGPoint originOfSclView = CGPointMake(headerWidth, 0);
        CGSize sizeOfSclView = CGSizeMake(width-headerWidth, height);
        _sclView = [[UIScrollView alloc] initWithFrame:CGRectMake(originOfSclView.x, originOfSclView.y, sizeOfSclView.width, sizeOfSclView.height)];
        [_sclView setContentSize:CGSizeMake(contentWidth, sizeOfSclView.height)];
        [_sclView setBounces:NO];
        [_sclView setDirectionalLockEnabled:YES];
        
        _contentTableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, _sclView.contentSize.width, _sclView.contentSize.height) style:UITableViewStylePlain];
    }
    [_contentTableView setShowsHorizontalScrollIndicator:NO];
    [_contentTableView setShowsVerticalScrollIndicator:NO];
    [_headerTableView setShowsHorizontalScrollIndicator:NO];
    [_headerTableView setShowsVerticalScrollIndicator:NO];
    [_sclView setShowsHorizontalScrollIndicator:NO];
    [_sclView setShowsVerticalScrollIndicator:NO];
    [_contentTableView setDelegate:self];
    [_contentTableView setDataSource:self];
    [_headerTableView setDelegate:self];
    [_headerTableView setDataSource:self];
    [_sclView addSubview:_contentTableView];
    [self addSubview:_headerTableView];
    [self addSubview:_sclView];
}

- (void)initAllData{
    int rows = [_dataSource numberOfRowsInMegaTableView:self];
    int columnsInContent = [_dataSource numberOfColumnsOfContentInMegaTableView:self];
    int columnsInHeader = [_dataSource numberOfColumnsOfHeaderInMegaTableView:self];
    _rowsInColumnsInContent = [[NSMutableArray alloc] initWithCapacity:columnsInContent];
    _columnsInRowsInContent = [[NSMutableArray alloc] initWithCapacity:rows];
    _rowsInColumnsInHeader = [[NSMutableArray alloc] initWithCapacity:columnsInHeader];
    _columnsInRowsInHeader = [[NSMutableArray alloc] initWithCapacity:rows];
    _titleInContent = [[NSMutableArray alloc] initWithCapacity:columnsInContent];
    _titleInHeader = [[NSMutableArray alloc] initWithCapacity:columnsInHeader];
    for (int i=0; i<rows; i++) {
        NSMutableArray *columnsForContent = [[NSMutableArray alloc] initWithCapacity:columnsInContent];
        NSMutableArray *columnsForHeader = [[NSMutableArray alloc] initWithCapacity:columnsInHeader];
        for (int j=0; j<columnsInContent; j++) {
            [columnsForContent addObject:[_dataSource megaTableView:self cellInContentAtRow:i column:j]];
        }
        for (int j=0; j<columnsInHeader; j++) {
            [columnsForHeader addObject:[_dataSource megaTableView:self cellInHeaderAtRow:i column:j]];
        }
        [_columnsInRowsInContent addObject:columnsForContent];
        [_columnsInRowsInHeader addObject:columnsForHeader];
    }
    for (int i=0; i<columnsInContent; i++) {
        NSMutableArray *rowsForContent = [[NSMutableArray alloc] initWithCapacity:rows];
        for (int j=0; j<rows; j++) {
            [rowsForContent addObject:[_dataSource megaTableView:self cellInContentAtRow:j column:i]];
        }
        [_rowsInColumnsInContent addObject:rowsForContent];
        [_titleInContent addObject:[_dataSource megaTableView:self titleInContentAtColumn:i]];
    }
    for (int i=0; i<columnsInHeader; i++) {
        NSMutableArray *rowsForHeader = [[NSMutableArray alloc] initWithCapacity:rows];
        for (int j=0; j<rows; j++) {
            [rowsForHeader addObject:[_dataSource megaTableView:self cellInHeaderAtRow:j column:i]];
        }
        [_rowsInColumnsInHeader addObject:rowsForHeader];
        [_titleInHeader addObject:[_dataSource megaTableView:self titleInHeaderAtColumn:i]];
    }
}

- (void)reloadAllData{
    int rows = [_dataSource numberOfRowsInMegaTableView:self];
    int columnsInContent = [_dataSource numberOfColumnsOfContentInMegaTableView:self];
    int columnsInHeader = [_dataSource numberOfColumnsOfHeaderInMegaTableView:self];
    [_rowsInColumnsInContent removeAllObjects];
    [_columnsInRowsInContent removeAllObjects];
    [_rowsInColumnsInHeader removeAllObjects];
    [_columnsInRowsInHeader removeAllObjects];
    [_titleInContent removeAllObjects];
    [_titleInHeader removeAllObjects];
    for (int i=0; i<rows; i++) {
        NSMutableArray *columnsForContent = [[NSMutableArray alloc] initWithCapacity:columnsInContent];
        NSMutableArray *columnsForHeader = [[NSMutableArray alloc] initWithCapacity:columnsInHeader];
        for (int j=0; j<columnsInContent; j++) {
            [columnsForContent addObject:[_dataSource megaTableView:self cellInContentAtRow:i column:j]];
        }
        for (int j=0; j<columnsInHeader; j++) {
            [columnsForHeader addObject:[_dataSource megaTableView:self cellInHeaderAtRow:i column:j]];
        }
        [_columnsInRowsInContent addObject:columnsForContent];
        [_columnsInRowsInHeader addObject:columnsForHeader];
    }
    for (int i=0; i<columnsInContent; i++) {
        NSMutableArray *rowsForContent = [[NSMutableArray alloc] initWithCapacity:rows];
        for (int j=0; j<rows; j++) {
            [rowsForContent addObject:[_dataSource megaTableView:self cellInContentAtRow:j column:i]];
        }
        [_rowsInColumnsInContent addObject:rowsForContent];
        [_titleInContent addObject:[_dataSource megaTableView:self titleInContentAtColumn:i]];
    }
    for (int i=0; i<columnsInHeader; i++) {
        NSMutableArray *rowsForHeader = [[NSMutableArray alloc] initWithCapacity:rows];
        for (int j=0; j<rows; j++) {
            [rowsForHeader addObject:[_dataSource megaTableView:self cellInHeaderAtRow:j column:i]];
        }
        [_rowsInColumnsInHeader addObject:rowsForHeader];
        [_titleInHeader addObject:[_dataSource megaTableView:self titleInHeaderAtColumn:i]];
    }
}

- (void)setColWidth{
    _colWidthForContent = [self setColWidthWithRowData:_rowsInColumnsInContent titleData:_titleInContent font:_cellFont];
    _colWidthForHeader = [self setColWidthWithRowData:_rowsInColumnsInHeader titleData:_titleInHeader font:_headerFont];
}

- (void)reloadColWidth{
    [_colWidthForContent removeAllObjects];
    [_colWidthForHeader removeAllObjects];
    [_colWidthForHeader addObjectsFromArray:[self setColWidthWithRowData:_rowsInColumnsInHeader titleData:_titleInHeader font:_headerFont]];
    [_colWidthForContent addObjectsFromArray:[self setColWidthWithRowData:_rowsInColumnsInContent titleData:_titleInContent font:_cellFont]];
}

- (void)drawTable{
    if ([_colWidthForHeader count]>0) {
        CGPoint position = CGPointMake(0.0f, 0.0f);
        for (int i=0; i<[_colWidthForHeader count]; i++) {
            position.x += [[_colWidthForHeader objectAtIndex:i]floatValue];
            [self addVerticalLineWithWidth:_lineWidth color:_lineColr atX:position.x];
        }
    }
    if ([_colWidthForContent count]>1) {
        CGPoint position = CGPointMake(0.0f, 0.0f);
        for (int i=0; i<[_colWidthForContent count]; i++) {
            position.x += [[_colWidthForContent objectAtIndex:i]floatValue];
            [_sclView addVerticalLineWithWidth:_lineWidth color:_lineColr atX:position.x];
        }
    }
}

- (void)setBGColor{
    if (respondsToCellBGColorInMegaTableView) {
        _cellBGColor = [_dataSource cellBGColorInMegaTableView:self];
    }
    if (respondsToHeaderBGColorInMegaTableView) {
        _headerBGColor = [_dataSource headerBGColorInMegatableView:self];
    }
    if (respondsToSectionBGColorInMegaTableView) {
        _sectionBGColor = [_dataSource sectionBGColorInMegaTableView:self];
    }
    if (respondsToTitleBGColorInMegaTableView) {
        _titleBGColor = [_dataSource titleBGColorInMegaTableView:self];
    }
}

- (void)setTextColor{
    if (respondsToCellTextColorInMegaTableView) {
        _cellTextColor = [_dataSource cellTextColorInMegaTableView:self];
    }
    if (respondsToHeaderTextColorInMegaTableView) {
        _headerTextColor = [_dataSource headerTextColorInMegatableView:self];
    }
    if (respondsToSectionTextColorInMegaTableView) {
        _sectionTextColor = [_dataSource sectionTextColorInMegaTableView:self];
    }
    if (respondsToTitleTextColorInMegaTableView) {
        _titleTextColor = [_dataSource titleTextColorInMegaTableView:self];
    }
}

- (void)setFont{
    if (respondsToCellFontInMegaTableView) {
        _cellFont = [_dataSource cellFontInMegaTableView:self];
    }
    if (respondsToHeaderFontInMegaTableView) {
        _headerFont = [_dataSource headerFontInMegaTableView:self];
    }
    if (respondsToSectionFontInMegaTableView) {
        _sectionFont = [_dataSource sectionFontInMegaTableView:self];
    }
    if (respondsToTitleFontInMegaTableView) {
        _titleFont = [_dataSource titleFontInMegaTableView:self];
    }
}

- (void)setLine{
    if (respondsToTableLineWidthInMegaTableView) {
        _lineWidth = [_dataSource tableLineWidthInMegatableView:self];
    }
    if (respondsToTableLineColorInMegaTableView) {
        _lineColr = [_dataSource tableLineColorInMegatableView:self];
    }
}

- (void)setSelectionColor{
    if (respondsToSelectionColorInMegatableView) {
        _selectionColor = [_dataSource selectionColorInMegatableView:self];
    }
}

- (void)setSection{
    if (respondsToSectionStructInMegatableView) {
        NSArray *sectionStruct = [_dataSource sectionStructInMegatableView:self];
        _sectionStartRow = [[sectionStruct objectAtIndex:0]intValue];
        _sectionStep = [[sectionStruct objectAtIndex:1]intValue];
    }
}

- (NSMutableArray *)setColWidthWithRowData:(NSMutableArray *)rowsInColumnData titleData:(NSMutableArray *)titleData font:(UIFont *)font{
    NSMutableArray *colWidthData = [[NSMutableArray alloc] initWithCapacity:[rowsInColumnData count]];
    //NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"length" ascending:YES];
    //NSArray *sortDescriptors = [NSArray arrayWithObject:sortDescriptor];
    for (int i=0; i<[rowsInColumnData count]; i++) {
        NSArray *rowsData = [rowsInColumnData objectAtIndex:i];
        NSString *maxStr = [titleData objectAtIndex:i];
        float maxLength = [self widthInColumnWithStr:maxStr font:_titleFont];
        for (NSString *str in rowsData) {
            float length = [self widthInColumnWithStr:str font:font];
            if (length>maxLength) {
                maxStr = str;
                maxLength = length;
            }
        }
        [colWidthData addObject:[NSNumber numberWithFloat:maxLength]];
    }
    return colWidthData;
}

- (CGFloat)widthInColumnWithStr:(NSString *)str font:(UIFont *)font{
    UILabel *label = [[UILabel alloc]init];
    [label setText:str];
    [label setFont:font];
    [label sizeToFit];
    return label.bounds.size.width+30;
}

- (UILabel *)cellWithStr:(NSString *)str font:(UIFont *)font textColor:(UIColor *)textColor bgColor:(UIColor *)bgColor frame:(CGRect)frame isGesture:(BOOL)isGesture{
    UILabel *label = [[UILabel alloc] initWithFrame:frame];
    [label setUserInteractionEnabled:isGesture];
    if (![str isEqualToString:@"nil"]) {
        [label setText:str];
    }
    UITapGestureRecognizer *recognizer;
    if (bgColor==_cellBGColor) {
        recognizer = [[UITapGestureRecognizer alloc]initWithTarget:self action:@selector(handleSTapInCellRecognizer:)];
        [label addGestureRecognizer:recognizer];
    }else if (bgColor==_sectionBGColor) {
        recognizer = [[UITapGestureRecognizer alloc]initWithTarget:self action:@selector(handleSTapInSectionRecongnizer:)];
        [label addGestureRecognizer:recognizer];
        
    }
    [label setTextColor:textColor];
    [label setFont:font];
    [label setBackgroundColor:bgColor];
    [label setTextAlignment:NSTextAlignmentCenter];
    return label;
}

- (void)handleSTapInCellRecognizer:(UITapGestureRecognizer *)recognizer{
    if ([recognizer.view.backgroundColor isEqual:_cellBGColor]) {
        [recognizer.view setBackgroundColor:_selectionColor];
    }else {
        [recognizer.view setBackgroundColor:_cellBGColor];
    }
}

- (void)handleSTapInSectionRecongnizer:(UITapGestureRecognizer *)recognizer{
    if ([recognizer.view.backgroundColor isEqual:_sectionBGColor]) {
        [recognizer.view setBackgroundColor:_selectionColor];
    }else {
        [recognizer.view setBackgroundColor:_sectionBGColor];
    }
}

#pragma mark - self protocol
- (void)configure{
    [self initAllData];
    [self setBGColor];
    [self setTextColor];
    [self setFont];
    [self setLine];
    [self setSection];
    [self setSelectionColor];
    [self setColWidth];
    [self initAllTableView];
    [self drawTable];
}

- (void)willMoveToSuperview:(UIView *)newSuperview{
    [self configure];
}

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect
{
    // Drawing code
}
*/

@end
